@@ -1,5 +1,5 @@
 /*
- * $Id: acountry.c,v 1.3 2008-05-08 22:14:17 bagder Exp $
+ * $Id: acountry.c,v 1.18 2009-11-10 18:41:03 yangtse Exp $
  *
  * IP-address/hostname to country converter.
  *
@@ -10,8 +10,8 @@
  *   CNAME = zz<CC>.countries.nerd.dk with address 127.0.x.y (ver 1) or
  *   CNAME = <a.b.c.d>.zz.countries.nerd.dk with address 127.0.x.y (ver 2)
  *
- * The 2 letter country code in <CC> and the ISO-3166 country
- * number in x.y (number = x*256 + y). Version 2 of the protocol is missing
+ * The 2 letter country code is in <CC> and the ISO-3166 country
+ * number is in x.y (number = x*256 + y). Version 2 of the protocol is missing
  * the <CC> number.
  *
  * Ref: http://countries.nerd.dk/more.html
@@ -33,7 +33,7 @@
  * without express or implied warranty.
  */
 
-#include "setup.h"
+#include "ares_setup.h"
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -47,9 +47,10 @@
 #include <strings.h>
 #endif
 
-#if defined(WIN32)
+#if defined(WIN32) && !defined(WATT32)
   #include <winsock.h>
 #else
+  #include <sys/socket.h>
   #include <arpa/inet.h>
   #include <netinet/in.h>
   #include <netdb.h>
@@ -60,6 +61,25 @@
 #include "inet_net_pton.h"
 #include "inet_ntop.h"
 
+#ifndef HAVE_STRDUP
+#  include "ares_strdup.h"
+#  define strdup(ptr) ares_strdup(ptr)
+#endif
+
+#ifndef HAVE_STRCASECMP
+#  include "ares_strcasecmp.h"
+#  define strcasecmp(p1,p2) ares_strcasecmp(p1,p2)
+#endif
+
+#ifndef HAVE_STRNCASECMP
+#  include "ares_strcasecmp.h"
+#  define strncasecmp(p1,p2,n) ares_strncasecmp(p1,p2,n)
+#endif
+
+#ifndef INADDR_NONE
+#define INADDR_NONE 0xffffffff
+#endif
+
 static const char *usage      = "acountry [-vh?] {host|addr} ...\n";
 static const char  nerd_fmt[] = "%u.%u.%u.%u.zz.countries.nerd.dk";
 static const char *nerd_ver1  = nerd_fmt + 14;
@@ -90,12 +110,19 @@ int main(int argc, char **argv)
   ares_channel channel;
   int    ch, status;
 
-#ifdef WIN32
+#if defined(WIN32) && !defined(WATT32)
   WORD wVersionRequested = MAKEWORD(USE_WINSOCK,USE_WINSOCK);
   WSADATA wsaData;
   WSAStartup(wVersionRequested, &wsaData);
 #endif
 
+  status = ares_library_init(ARES_LIB_INIT_ALL);
+  if (status != ARES_SUCCESS)
+    {
+      fprintf(stderr, "ares_library_init: %s\n", ares_strerror(status));
+      return 1;
+    }
+
   while ((ch = ares_getopt(argc, argv, "dvh?")) != -1)
     switch (ch)
       {
@@ -158,7 +185,9 @@ int main(int argc, char **argv)
   wait_ares(channel);
   ares_destroy(channel);
 
-#ifdef WIN32
+  ares_library_cleanup();
+
+#if defined(WIN32) && !defined(WATT32)
   WSACleanup();
 #endif
 
@@ -170,7 +199,7 @@ int main(int argc, char **argv)
  */
 static void wait_ares(ares_channel channel)
 {
-  while (1)
+  for (;;)
     {
       struct timeval *tvp, tv;
       fd_set read_fds, write_fds;
@@ -236,7 +265,7 @@ struct search_list {
        const char *long_name;      /* normal country name */
      };
 
-const struct search_list *list_lookup(int number, const struct search_list *list, int num)
+static const struct search_list *list_lookup(int number, const struct search_list *list, int num)
 {
   while (num > 0 && list->long_name)
     {
@@ -504,7 +533,8 @@ static int is_addr(char *str, char **end)
 {
   int a0, a1, a2, a3, num, rc = 0, length = 0;
 
-  if ((num = sscanf(str,"%3d.%3d.%3d.%3d%n",&a0,&a1,&a2,&a3,&length)) == 4 &&
+  num = sscanf(str,"%3d.%3d.%3d.%3d%n",&a0,&a1,&a2,&a3,&length);
+  if( (num == 4) &&
       BYTE_OK(a0) && BYTE_OK(a1) && BYTE_OK(a2) && BYTE_OK(a3) &&
       length >= (3+4))
     {
@@ -526,12 +556,13 @@ static void find_country_from_cname(const char *cname, struct in_addr addr)
   const struct search_list *country;
   char  ccode_A2[3], *ccopy, *dot_4;
   int   cnumber, z0, z1, ver_1, ver_2;
-  u_long ip;
+  unsigned long ip;
 
   ip = ntohl(addr.s_addr);
   z0 = tolower(cname[0]);
   z1 = tolower(cname[1]);
   ccopy = strdup(cname);
+  dot_4 = NULL;
 
   ver_1 = (z0 == 'z' && z1 == 'z' && !strcasecmp(cname+4,nerd_ver1));
   ver_2 = (is_addr(ccopy,&dot_4) && !strcasecmp(dot_4,nerd_ver2));
@@ -563,8 +594,8 @@ static void find_country_from_cname(const char *cname, struct in_addr addr)
 
   if (ver_1)
     {
-      ccode_A2[0] = tolower(cname[2]);
-      ccode_A2[1] = tolower(cname[3]);
+      ccode_A2[0] = (char)tolower(cname[2]);
+      ccode_A2[1] = (char)tolower(cname[3]);
       ccode_A2[2] = '\0';
     }
   else
@@ -581,9 +612,14 @@ static void find_country_from_cname(const char *cname, struct in_addr addr)
     printf("Name for country-number %d not found.\n", cnumber);
   else
     {
-      if (ver_1 && *(unsigned short*)&country->short_name != *(unsigned*)&ccode_A2)
-        printf("short-name mismatch; %s vs %s\n", country->short_name, ccode_A2);
-
+      if (ver_1)
+        {
+          if ((country->short_name[0] != ccode_A2[0]) ||
+              (country->short_name[1] != ccode_A2[1]) ||
+              (country->short_name[2] != ccode_A2[2]))
+            printf("short-name mismatch; %s vs %s\n",
+                   country->short_name, ccode_A2);
+        }
       printf("%s (%s), number %d.\n",
              country->long_name, country->short_name, cnumber);
     }
