@@ -1,4 +1,4 @@
-/* $Id: ares_free_string.c,v 1.6 2006-07-22 15:37:10 giva Exp $ */
+/* $Id: ares_free_string.c,v 1.7 2009-11-02 11:55:53 yangtse Exp $ */
 
 /* Copyright 2000 by the Massachusetts Institute of Technology.
  *
@@ -15,7 +15,7 @@
  * without express or implied warranty.
  */
 
-#include "setup.h"
+#include "ares_setup.h"
 #include <stdlib.h>
 #include "ares.h"
 #include "ares_private.h"
